#include "chr_btreegui_latex.h"

#include <stdio.h>
#include <stdlib.h>
#include <sstream>

chr_btreegui_latex::chr_btreegui_latex()
{
}

chr_btreegui_latex::~chr_btreegui_latex()
{
}

std::string tohex(const unsigned char * value, size_t size)
{
  std::ostringstream ret;
  ret << std::hex;
  for(size_t i = 0; i < size; i++)
  {
    char tmp[3];
    sprintf(tmp, "%02X", (unsigned short)value[i]);
    ret << tmp;
  }
  return(ret.str());
}

void chr_btreegui_latex::print_inode(chr_inode * inode)
{
  chr_lsn lsn;
  bool kmininf;
  unsigned char * kmin = NULL;
  bool kmaxinf;
  unsigned char * kmax = NULL;
  size_t key_size;
  unsigned int nb_keys;
  unsigned char * keys = NULL;
  chr_ptr * ptrs = NULL;
  inode->get_state(&lsn, &kmininf, &kmin, &kmaxinf, &kmax, &key_size,
      &nb_keys, &keys, &ptrs);
  fprintf(stdout, "[.\\chn{%lld[", lsn);
  if(kmininf)
  {
    fprintf(stdout, "-inf");
  } 
  else
  {
    fprintf(stdout, "%s", tohex(kmin, key_size).c_str());
  } 
  fprintf(stdout, ",");
  if(kmaxinf)
  {
    fprintf(stdout, "+inf");
  } 
  else
  {
    fprintf(stdout, "%s", tohex(kmax, key_size).c_str());
  } 
  fprintf(stdout, "[}{%d}{", nb_keys);
  for(unsigned int i =0; i < nb_keys; i++)
  {
    fprintf(stdout, "%s", tohex(&(keys[i * key_size]), key_size).c_str());
    if(i != nb_keys - 1)
    {
      fprintf(stdout, "&");
    } 
  } 
  fprintf(stdout, "} ");
  for(unsigned int i =0; i <= nb_keys; i++)
  {
    print_ptr(ptrs[i]);
  }
  fprintf(stdout, "] ");
}

void chr_btreegui_latex::print_tree(chr_node * root)
{
  fprintf(stdout, "\\Tree ");
  if(root == NULL)
  {
    fprintf(stdout, "[.NULL ]");
  }
  else
  {
    print_node(root);
  }
  fprintf(stdout, "\n");
}

void chr_btreegui_latex::print_node(chr_node * node)
{
  if(node->is_leaf())
  {
    print_leaf(node->leaf());
  }
  else
  {
    print_inode(node->inode());
  }
}

void chr_btreegui_latex::print_leaf(chr_leaf * leaf)
{
  chr_lsn lsn;
  bool kmininf;
  unsigned char * kmin = NULL;
  bool kmaxinf;
  unsigned char * kmax = NULL;
  size_t key_size;
  size_t value_size;
  unsigned int nb_keys;
  unsigned char * keys = NULL;
  unsigned char * values = NULL;
  leaf->get_state(&lsn, &kmininf, &kmin, &kmaxinf, &kmax, &key_size,
      &value_size, &nb_keys, &keys, &values);
  fprintf(stdout, "\\chf{%lld[", lsn);
  if(kmininf)
  {
    fprintf(stdout, "-inf");
  } 
  else
  {
    fprintf(stdout, "%s", tohex(kmin, key_size).c_str());
  } 
  fprintf(stdout, ",");
  if(kmaxinf)
  {
    fprintf(stdout, "+inf");
  } 
  else
  {
    fprintf(stdout, "%s", tohex(kmax, key_size).c_str());
  } 
  fprintf(stdout, "[}{%d}{", nb_keys);
  for(int i = 0; i < (int)nb_keys - 1; i++)
  {
    fprintf(stdout, "%s", tohex(&(keys[i * key_size]), key_size).c_str());
    fprintf(stdout, "&");
  }
  if(nb_keys > 0)
  {
    fprintf(stdout, "%s", tohex(&(keys[(nb_keys - 1) * key_size]), key_size).c_str());
  }
  fprintf(stdout, "}{");
  for(int i = 0; i < (int)nb_keys - 1; i++)
  {
    fprintf(stdout, "%s",
        tohex(&(values[i * value_size]), key_size).c_str());
    fprintf(stdout, "&");
  }
  if(nb_keys > 0)
  {
    fprintf(stdout, "%s",
        tohex(&(values[(nb_keys - 1) * value_size]), key_size).c_str());
  }
  fprintf(stdout, "} ");
}

void chr_btreegui_latex::print_ptr(chr_ptr ptr)
{
  bool load = false;
  if(ptr.is_memory())
  {
    print_node(ptr.get_node());
  }
  else
  {
    if(!load)
    {
      /* do not load nodes from flash */
      fprintf(stdout, "\\chl{%lld} ", ptr.get_lsn());
    }
    else
    {
      /* load nodes from flash 
      const chr_flash * flash = table->get_flash();
      unsigned char * buf = (unsigned char *)malloc(CHR_BLOCK_SIZE);
      assert(buf != NULL);
      flash->read(get_lsn(), buf);
      if(chr_parse::is_leaf(buf))
      {
        unsigned int nb_keys = chr_parse::leaf_nb_keys(buf);
        size_t key_size = table->get_key_size();
        size_t value_size = table->get_value_size();
        fprintf(stdout, "\\chf{%lld*}{%d}{", get_lsn(), nb_keys);
        for(int i = 0; i < (int)nb_keys - 1; i++)
        {
          table->print_key(chr_parse::
              leaf_key(buf, i, key_size, value_size));
          fprintf(stdout, "&");
        }
        if(nb_keys > 0)
        {
          table->print_key(chr_parse::
              leaf_key(buf, nb_keys - 1, key_size, value_size));
        }
        fprintf(stdout, "}{");
        for(int i = 0; i < (int)nb_keys - 1; i++)
        {
          table->print_value(chr_parse::
              leaf_value(buf, i, key_size, value_size));
          fprintf(stdout, "&");
        }
        if(nb_keys > 0)
        {
          table->print_value(chr_parse::
              leaf_value(buf, nb_keys - 1, key_size, value_size));
        }
        fprintf(stdout, "} ");
      }
      else
      {
        unsigned int nb_keys = chr_parse::inode_nb_keys(buf);
        size_t key_size = table->get_key_size();
        fprintf(stdout, "[.\\chn{%lld*}{%d}{", get_lsn(), nb_keys);
        for(unsigned int i =0; i < nb_keys; i++)
        {
          table->print_key(chr_parse::inode_key(buf, i, key_size));
          if(i != nb_keys - 1)
          {
            fprintf(stdout, "&");
          }
        }
        fprintf(stdout, "} ");
        for(unsigned int i =0; i <= nb_keys; i++)
        {
          chr_ptr son = chr_parse::inode_ptr(buf, i, key_size);
          son.print(table, load);
        }
        fprintf(stdout, "] ");
      }
      free(buf);
      */
    }
  }
}

