#ifndef CHR_INODE_H
#define CHR_INODE_H

#include "chr_common.h"

#include "chr_flash.h"
#include "chr_table.h"
#include "chr_ptr.h"
#include "chr_node.h"

class chr_inode :  public chr_node
{
public:
  chr_inode(const size_t key_size, chr_inode * parent, chr_table * table,
      chr_flash * flash, bool kmininf, const unsigned char * kmin,
      bool kmaxinf, const unsigned char * kmax, chr_lsn lsn,
      const unsigned char * buf);
  ~chr_inode();

  void init_son(const chr_ptr ptr);
  int insert_left(const unsigned char * key, const chr_ptr ptr);
  int insert_right(const unsigned char * key, const chr_ptr ptr);
  int del(const unsigned char * key);
  chr_node * son(const unsigned char * key);
  chr_node * nextson(const unsigned char * key);
  chr_node * prevson(const unsigned char * key);
  int update(const unsigned char * key, const chr_ptr ptr);
  chr_leaf * search_leaf(const unsigned char * key, chr_pos_type type,
      bool minusinf, const unsigned char * kmin,
      bool plusinf, const unsigned char * kmax);
  void drop();

  bool is_leaf();
  chr_leaf * leaf();
  chr_inode * inode();

  void dbug_check(bool kmininf, const unsigned char * kmin, bool kmaxinf,
      const unsigned char * kmax);

  void get_state(chr_lsn * lsn, bool * kmininf, unsigned char ** kmin,
      bool * kmaxinf, unsigned char ** kmax, size_t * key_size, unsigned int * nb_keys, unsigned char ** keys, chr_ptr ** ptrs);

private:
  bool dirty;
  chr_lsn lsn;

  chr_ptr * ptrs;

  chr_ptr * ptr(int i);
  int flush();
  void to_sector(unsigned char * buf);
  int split();
  void load_ptr(unsigned int i);

  chr_inode & operator=(const chr_inode &);
  chr_inode(const chr_inode &);
};

#endif /* CHR_NODE_H */

