#include "chr_node.h"

#include <stdlib.h>
#include <assert.h>
#include <string.h>

#include "chr_common.h"

chr_node::chr_node(size_t key_size, unsigned int keys_per_node,
    chr_inode * parent, chr_table * table, chr_flash * flash,
    bool kmininf, const unsigned char * kmin,
    bool kmaxinf, const unsigned char * kmax)
: key_size(key_size),
  keys_per_node(keys_per_node),
  parent(parent),
  table(table),
  flash(flash),
  keys(NULL),
  bind_count(0),
  kmin(NULL),
  kmax(NULL)
{
  assert(keys_per_node > 1);
  assert(table != NULL);
  assert(flash != NULL);
  this->keys = (unsigned char *)malloc(key_size * keys_per_node);
  this->kmin = (unsigned char *)malloc(key_size);
  this->kmax = (unsigned char *)malloc(key_size);
  if(
      this->keys == NULL ||
      this->kmin == NULL ||
      this->kmax == NULL
    )   
  {   
    free(this->keys);
    free(this->kmin);
    free(this->kmax);
    throw std::bad_alloc();
  } 
  set_kmin(kmininf, kmin);
  set_kmax(kmaxinf, kmax);
}

chr_node::~chr_node()
{
  free(keys);
  free(kmin);
  free(kmax);
}

/* gives pos[k], k = max(k <= key), -1 if all k > key */
int chr_node::search(int * pos, const unsigned char * k) const
{
  /* assert(nb_keys >= 1); actually, nb_keys == 0 is OK */
  int min = 0;
  int max = nb_keys - 1;
  int mid;
  int cmp;
  while(min <= max)
  {
    mid = (min + max) / 2;
    cmp = comp(k, key(mid));
    if(cmp == 0)
    {
      *pos = mid;
      return(CHR_FOUND);
    }
    if(cmp > 0)
    {
      min = mid + 1;
    }
    else
    {
      max = mid - 1;
    }
  }
  *pos = max;
  return(CHR_NOTFOUND);
}

void chr_node::unbind()
{
  assert(bind_count > 0);
  bind_count--;
  if(bind_count == 0)
  {
    delete this;
  }
}

void chr_node::update_range(bool minusinf, const unsigned char * kmin,
    bool plusinf, const unsigned char * kmax)
{
  set_kmin(minusinf, kmin);
  set_kmax(plusinf, kmax);
}

void chr_node::set_kmin(bool inf, const unsigned char * kmin)
{
  kmininf = inf;
  if(!inf)
  {
    assert(kmin != NULL);
    memcpy(this->kmin, kmin, key_size);
  }
  else
  {
    memset(this->kmin, 0, key_size);
  }
}

void chr_node::set_kmax(bool inf, const unsigned char * kmax)
{
  kmaxinf = inf;
  if(!inf)
  {
    assert(kmax != NULL);
    memcpy(this->kmax, kmax, key_size);
  }
  else
  {
    memset(this->kmax, 255, key_size);
  }
}

