#ifndef CHR_NODE_H
#define CHR_NODE_H

#include "chr_common.h"

#include <stdlib.h>
#include <assert.h>
#include <string.h>

#include "chr_flash.h"

class chr_inode;
class chr_leaf;
class chr_table;

class chr_node
{
public:
  chr_node(size_t key_size, unsigned int keys_per_node,
      chr_inode * parent, chr_table * table, chr_flash * flash,
      bool kmininf, const unsigned char * kmin,
      bool kmaxinf, const unsigned char * kmax);
  virtual ~chr_node();
  virtual bool is_leaf() = 0;
  virtual chr_leaf * leaf() = 0;
  virtual chr_inode * inode() = 0;
  unsigned char * key(unsigned int i);
  const unsigned char * key(unsigned int i) const;
  virtual void drop() = 0;

  void bind();
  void unbind();

  virtual void dbug_check(bool kmininf, const unsigned char * kmin,
      bool kmaxinf, const unsigned char * kmax) = 0;

  void update_range(bool minusinf, const unsigned char * kmin,
      bool plusinf, const unsigned char * kmax);
protected:
  int comp(const unsigned char * k1, const unsigned char * k2) const;
  int search(int * pos, const unsigned char * k) const;
  
  void set_kmin(bool inf, const unsigned char * kmin);
  /* kmin <= k in leaf */
  void set_kmax(bool inf, const unsigned char * kmax);
  /* kmax > k in leaf */
  bool in_range(const unsigned char * key);
  
  const size_t key_size;
  const unsigned int keys_per_node;
public:
  chr_inode * parent;
protected:
  unsigned int nb_keys;
  chr_table * const table;
  chr_flash * const flash;

  unsigned char * keys;
public:
  /* bind_count is public for debuging purposes */
  int bind_count;
protected:
  unsigned char * kmin;
  bool kmininf;
  unsigned char * kmax;
  bool kmaxinf;
private:
  chr_node(const chr_node &);
  chr_node & operator=(const chr_node &);
};


inline const unsigned char * chr_node::key(unsigned int i) const
{
  assert(i < keys_per_node);
  return(&(keys[i * key_size]));
}

inline unsigned char * chr_node::key(unsigned int i)
{
  assert(i < keys_per_node);
  return(&(keys[i * key_size]));
}

inline int chr_node::
comp(const unsigned char * k1, const unsigned char * k2) const
{
  return(memcmp(k1, k2, key_size));
}

inline void chr_node::bind()
{
  bind_count++;
}

inline bool chr_node::in_range(const unsigned char * k)
{
  /* if(k >= kmax || k < kmin) */
  if((!kmaxinf && comp(k, kmax) >= 0) ||
      (!kmininf && comp(k, kmin) < 0))
  {
    return(false);
  }
  /* kmin <= k < kmax */
  return(true);
}

#endif /* CHR_NODE_H */

