#include "chr_list.h"
#include "chr_leaf.h"
#include "chr_chronos.h"
#include "chr_flashgui_text.h"

#include <iostream>
#include <sstream>
#include <arpa/inet.h>

#define CHR_DEVICE "flash.dummy"
#define CHR_FOLDER "chr_desc/"

using namespace std;

typedef unsigned char * k_t;

/*
  test case: 
  insert 4, 7
  pop x2 (4, 7)
  insert 4
  pop x1 (4)
  insert 10, 4, 7, 2, 3, 11
  pop x3 (2, 3, 4)
  insert 2
  pop x4 (7, 10, 11, 2)
  insert 5, 4, 6, 1
  pop x1 (1)
*/
void check_chr_list()
{
  cout << "checking chr_list... " << endl;
  chr_list<chr_lsn, NULL_LSN> * lst = new chr_list<chr_lsn, NULL_LSN>();
  lst->insert(4);
  lst->insert(7);

  assert(lst->pop() == 4);
  assert(lst->pop() == 7);

  lst->insert(4);
  assert(lst->pop() == 4);
  
  lst->insert(10);
  lst->insert(4);
  lst->insert(7);
  lst->insert(2);
  lst->insert(3);
  lst->insert(11);

  assert(lst->pop() == 2);
  assert(lst->pop() == 3);
  assert(lst->pop() == 4);

  lst->insert(2);

  assert(lst->pop() == 7);
  assert(lst->pop() == 10);
  assert(lst->pop() == 11);
  assert(lst->pop() == 2);

  lst->insert(5);
  lst->insert(4);
  lst->insert(6);
  lst->insert(1);

  assert(lst->pop() == 1);

  delete lst;
  cout << "checking chr_list OK" << endl;
}

std::string format_k(const unsigned char * k, size_t s)
{
  uint32_t key = htonl(*((uint32_t *)k));
  ostringstream out;
  out << key;
  return out.str();
}

std::string format_v(const unsigned char * v, size_t s)
{
  uint32_t val = *((uint32_t *)v);
  ostringstream out;
  out << val;
  return out.str();
}

/*
  test case:
  keys: CHAR(3)
  values: INT
  insert <"bb",1>
  insert <"bb","> -> DUPKEY error
  key(0) == "bb"
  value(0) == 1
  insert <"aa",2>
  key(1) == "bb"
  value(1) == 1
  key(0) == "aa"
  value(0) == 2
  insert <"cc",3>
  key(2) == "cc"
  value(2) == 3
  update <"ab",4> -> NOTFOUND error
  update <"bb",4>
  value(1) == 4
  delete("ab") -> NOTFOUND error
  delete("bb")
  key(0) == "aa"
  value(0) == 2
  key(1) == "cc"
  value(1) == 3
  TBD range_checking
*/
void check_chr_leaf()
{
  cout << "checking chr_leaf... " << endl;

  chr_leaf * leaf = new chr_leaf(3, sizeof(int), NULL, NULL, NULL, true,
      NULL, true, NULL, NULL_LSN, NULL);
  unsigned char kin[3];
  int vin;
  memcpy(&kin, "bb", 3);
  vin = 1;
  leaf->insert(kin, (k_t)&vin);
  assert(leaf->insert(kin, (k_t)&vin) == CHR_DUPKEY);
  assert(strcmp((char *)leaf->key(0), "bb") == 0);
  assert(*((int *)leaf->value(0)) == 1);
  memcpy(&kin, "aa", 3);
  vin = 2;
  leaf->insert(kin, (k_t)&vin);
  assert(strcmp((char *)leaf->key(1), "bb") == 0);
  assert(*((int *)leaf->value(1)) == 1);
  assert(strcmp((char *)leaf->key(0), "aa") == 0);
  assert(*((int *)leaf->value(0)) == 2);
  
  memcpy(&kin, "cc", 3);
  vin = 3;
  leaf->insert(kin, (k_t)&vin);
  assert(strcmp((char *)leaf->key(2), "cc") == 0);
  assert(*((int *)leaf->value(2)) == 3);

  vin = 4;
  assert(leaf->update((k_t)"ab", (k_t)&vin) == CHR_NOTFOUND);
  
  leaf->update((k_t)"bb", (k_t)&vin);
  assert(*((int *)leaf->value(1)) == 4);

  assert(leaf->del((k_t)"ab") == CHR_NOTFOUND);
  leaf->del((k_t)"bb");
  assert(strcmp((char *)leaf->key(0), "aa") == 0);
  assert(*((int *)leaf->value(0)) == 2);
  assert(strcmp((char *)leaf->key(1), "cc") == 0);
  assert(*((int *)leaf->value(1)) == 3);
  /* leaf->print(print_key, print_value); */

  int v = 0;
  unsigned char * vout = (unsigned char *)(&v);
  unsigned char kout[3];
  int pos = leaf->read_exact((k_t)"aa", vout);
  assert(v == 2);
  pos = leaf->readnext((k_t)"aa", kout, vout, pos);
  assert(strcmp((char *)kout, "cc") == 0);
  assert(v == 3);
  assert(leaf->readnext(kout, kout, vout, pos) == CHR_NEXTLEAF);

  delete leaf;

  cout << "checking chr_leaf OK" << endl;
}

void check_chr_chronos()
{
  cout << "checking chr_chronos... " << endl;

  chr_chronos * chronos = new chr_chronos(CHR_DEVICE, CHR_FOLDER);

  chronos->drop_table("footable");
  chronos->drop_table("footable2");
  assert(chronos->create_table("footable", 3, 4) == CHR_OK);
  assert(chronos->create_table("footable", 3, 4) == CHR_ERR);
  assert(chronos->drop_table("footable") == CHR_OK);
  assert(chronos->drop_table("footable") == CHR_NOTFOUND);
  assert(chronos->create_table("footable", 3, 4) == CHR_OK);
  assert(chronos->create_table("footable2", 5, 6) == CHR_OK);
  assert(chronos->drop_table("footable") == CHR_OK);
  assert(chronos->drop_table("footable2") == CHR_OK);
  assert(chronos->create_table("footable", 3, 4) == CHR_OK);
  assert(chronos->create_table("footable2", 5, 6) == CHR_OK);
  assert(chronos->drop_table("footable2") == CHR_OK);
  assert(chronos->drop_table("footable") == CHR_OK);
  assert(chronos->create_table("footable2", 5, 6) == CHR_OK);
  assert(chronos->create_table("footable", 3, 4) == CHR_OK);

  chr_table_ctx * ctx = chronos->open_table("footable");
  assert(ctx != NULL);

  chr_cursor * cur = ctx->open_cursor(NULL, CHR_FIRST);

  assert(cur->insert((k_t)"aa", (k_t)"aa1") == CHR_OK);
  assert(cur->insert((k_t)"aa", (k_t)"aa2") == CHR_DUPKEY);

  ctx->close_cursor(cur);

  assert(chronos->close_table(ctx) == CHR_OK);

  delete chronos;

  cout << "checking chr_chronos OK" << endl;
}

void check_chr_chronos_rand()
{
  cout << "checking chr_chronos (rand)..." << endl;
  chr_chronos * chr = new chr_chronos(CHR_DEVICE, CHR_FOLDER);
  chr->drop_table("test");
  assert(chr->create_table("test", sizeof(uint64_t), sizeof(uint64_t)) == CHR_OK);
  chr_table_ctx * tbl = chr->open_table("test");
  assert(tbl != NULL);
  chr_cursor * cur = tbl->open_cursor(NULL, CHR_FIRST);
  srand(123);

  uint64_t dup = 0;
  uint64_t nb = 1000000LL;
  for(uint64_t i = 0; i < nb; i++)
  {
    //long long int val = rand() % (nb * 10) + 1;
    uint64_t val = i + 1;
    uint64_t key = htobe64(val);
    int res = cur->insert((k_t)&key, (k_t)&val);
    if (res == CHR_DUPKEY)
    {
      dup++;
    } else {
      assert(res == CHR_OK);
    }
  }

  tbl->close_cursor(cur);
  cur = tbl->open_cursor(NULL, CHR_FIRST);
  uint64_t key;
  uint64_t val;
  while (cur->read_next((k_t)&key, (k_t)&val) != CHR_EOF) {
    key = be64toh(key);
    //cout << "read: " << key << ";" << val << endl;
    assert(key == val);
    nb--;
    tbl->table->print();
  }

  //cout << "nb: " << nb << " dup: " << dup << endl;
  assert(nb == dup);
  
  tbl->close_cursor(cur);
  assert(chr->close_table(tbl) == CHR_OK);
  delete chr;

  cout << "checking chr_chronos (rand) OK" << endl;
}

#define PRINT_CHR_TREE false
#define PLOAD_CHR_TREE true

static const char * chr_ok_s = "CHR_OK";
static const char * chr_dupkey_s = "CHR_DUPKEY";
static const char * chr_unknown_s = "unknown_error";

const char * error_str(int code)
{
  if(code == CHR_OK)
  {
    return(chr_ok_s);
  }
  if(code == CHR_DUPKEY)
  {
    return(chr_dupkey_s);
  }
  return(chr_unknown_s);
}

void insert_and_check(chr_table_ctx * ctx, chr_cursor * cur, uint32_t k,
    uint32_t v, int stat)
{
  uint32_t kn = htonl(k);
  //printf("inserting %d:\n", k);
  int ret = cur->insert((k_t)&kn, (k_t)&v);
  //printf("insert(%d) returned %s (%d)\n", k, error_str(ret), ret);
  assert(ret == stat || ret == CHR_DUPKEY);
}

void check_chr_chronos_tree()
{
  cout << "checking chr_chronos_tree... " << endl;

  chr_chronos * chronos = new chr_chronos(CHR_DEVICE, CHR_FOLDER);

  chronos->drop_table("footable");
  assert(chronos->create_table("footable", sizeof(uint32_t),
        sizeof(uint32_t)) == CHR_OK);

  chr_table_ctx * ctx = chronos->open_table("footable");
  assert(ctx != NULL);

  chr_cursor * cur = ctx->open_cursor(NULL, CHR_FIRST);

  insert_and_check(ctx, cur, 1, 1, CHR_OK);
  insert_and_check(ctx, cur, 2, 2, CHR_OK);
  
  srand(123);

  for(int i=3; i<100; i++)
  {
    insert_and_check(ctx, cur, rand() % 1000, i, CHR_OK);
  }

  ctx->close_cursor(cur);

  assert(chronos->close_table(ctx) == CHR_OK);

  delete chronos;

  cout << "checking chr_chronos_tree OK" << endl;
}

void check_chr_flashgui_text()
{
  cout << "checking chr_flashgui_text" << endl;
  chr_flashgui * gui = new chr_flashgui_text();
//  chr_flashgui * gui = new CHR_FLASH_GUI();

  delete gui;
  cout << "checking chr_flashgui_text OK" << endl;
}

int main(int argc, const char * argv[])
{
  int i = 0;
  assert(i = 1);
  if(i == 0)
  {
    cout << "error: assertions are disabled" << endl;
    exit(EXIT_FAILURE);
  }
  check_chr_list();
  //check_chr_leaf();
  check_chr_chronos();
  check_chr_chronos_tree();
  //check_chr_flashgui_text();
  check_chr_chronos_rand();
  return(0);
}

